#!/usr/bin/env python2.7

# Copyright (C) 2012 Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys

# pylint: disable-msg=F0401
import presolve
from packagekit import backend


class Backend(backend.PackageKitBaseBackend):

    def __init__(self, args):
        backend.PackageKitBaseBackend.__init__(self, args)

    def simulate_install_packages(self, package_ids):
        self._install_packages(False, package_ids, simulate=True)

    def install_packages(self, only_trusted, package_ids):
        self._install_packages(only_trusted, package_ids, simulate=False)

    def resolve(self, filters, packages):
        self.status(backend.STATUS_QUERY)
        self.allow_cancel(False)
        self.percentage(None)

        for i in packages:
            self._resolve(i, filters)

    def _install_packages(self, only_trusted, package_ids, simulate=False):
        self.allow_cancel(False)
        self.percentage(None)

        if simulate:
            return

        self.status(backend.STATUS_INSTALL)
        try:
            for i in package_ids:
                self._install(backend.split_package_id(i)[0])
        except Exception, error:
            self.error(backend.ERROR_UNKNOWN, error)

    def _resolve(self, package, filters):
        info = presolve.resolve(package)
        if info is None:
            self.error(backend.ERROR_PACKAGE_NOT_FOUND,
                    'Package %r was not found', package)

        if info['installed']:
            status = backend.INFO_INSTALLED
        else:
            status = backend.INFO_AVAILABLE

        if filters and 'none' not in filters:
            if backend.FILTER_INSTALLED in filters and \
                    status != backend.INFO_INSTALLED:
                return
            if backend.FILTER_NOT_INSTALLED in filters and \
                    status == backend.INFO_INSTALLED:
                return

        package_id = backend.get_package_id(
                package, info['version'], info['arch'], '')
        self.package(package_id, status, '')

    def _install(self, package):
        presolve.install(package)


if __name__ == '__main__':
    Backend('').dispatcher(sys.argv[1:])
